package info.textgrid.middleware;

import java.io.IOException;
import java.io.StringReader;
import java.text.ParseException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.classicmayan.tools.ClassicMayanMetsMods;
import org.elasticsearch.action.get.GetRequest;
import org.elasticsearch.action.get.GetResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.common.Strings;
import org.elasticsearch.search.fetch.subphase.FetchSourceContext;
import org.json.JSONObject;
import org.springframework.stereotype.Component;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import info.textgrid.middleware.oaipmh.GetRecordType;
import info.textgrid.middleware.oaipmh.HeaderType;
import info.textgrid.middleware.oaipmh.MetadataType;
import info.textgrid.middleware.oaipmh.RecordType;

/**
 * @author Maximilian Brodhun, SUB Göttingen
 * @author Stefan E. Funk, SUB Göttingen
 * @version 2022-08-17
 * @since 2019-03-12
 */
@Component
public class RecordDelivererIDIOM extends RecordDelivererAbstract {

  private String dateOfLastOblectModification;
  private String objectType;

  RecordType record = new RecordType();
  ClassicMayanMetsMods metsmods;

  /**
   * @param textgrid
   * @param dariah
   */
  public RecordDelivererIDIOM(boolean textgrid, boolean dariah) {
    super(textgrid, dariah);
  }

  /**
   * <p>
   * ElasticSearch request in non-public index to get the datestamps for the object creation and
   * last modification date in TextGridRep.
   * </p>
   * 
   * @param idInDatabase
   * @throws ParseException
   * @throws IOException
   */
  public void setDatestamps(String idInDatabase) throws ParseException, IOException {

    String changedId = idInDatabase;

    String[] includes = new String[] {TGConstants.CREATED, TGConstants.MODIFIED_FIELD};
    String[] excludes = Strings.EMPTY_ARRAY;
    FetchSourceContext fetchSourceContext = new FetchSourceContext(true, includes, excludes);

    if (idInDatabase.startsWith("textgrid:")) {
      changedId = idInDatabase.substring("textgrid:".length());
    }

    if (!idInDatabase.contains(".0")) {
      // changedId = changedId.substring(0,changedId.length() - 1) + "0";
      changedId = changedId + ".0";
    }

    GetRequest getRequest =
        new GetRequest("textgrid-nonpublic", OAI_ESClient.getEsType(), changedId)
            .fetchSourceContext(fetchSourceContext);

    GetResponse objectInDatabase = null;
    try {
      objectInDatabase = OAI_ESClient.getEsClient().get(getRequest, RequestOptions.DEFAULT);
    } catch (IOException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    JSONObject json = new JSONObject();
    String[] fields = {TGConstants.CREATED, TGConstants.MODIFIED_FIELD, "notes"};
    json = new JSONObject(OAIPMHUtilities
        .getRcordByIDFromElasticSearch("textgrid-nonpublic", changedId, fields, Strings.EMPTY_ARRAY)
        .getSource());

    this.dateOfObjectCreation =
        OAIPMHUtilities.datestampAsString(OAIPMHUtilities.fieldLoader(json, TGConstants.CREATED));

    this.dateOfLastOblectModification = OAIPMHUtilities
        .datestampAsString(OAIPMHUtilities.fieldLoader(json, TGConstants.MODIFIED_FIELD));

    this.setObjectType(OAIPMHUtilities.fieldLoader(json, "notes"));
  }

  /**
   * <p>
   * Building the record XML object for the OAI-PMH response.
   * </p>
   * 
   * @throws ParseException
   * @throws IOException
   */
  @Override
  public GetRecordType getRecordById(String id) throws ParseException, IOException {

    GetRecordType grt = new GetRecordType();

    setDatestamps(id);
    String textgridBaseURI_FromID;
    if (!id.startsWith("textgrid:")) {
      textgridBaseURI_FromID = "textgrid:" + id.replace(".0", "");
    } else {
      textgridBaseURI_FromID = id;
    }
    if (this.getObjectType().equals("ARTEFACT")) {
      this.metsmods = new ClassicMayanMetsMods(
          textgridBaseURI_FromID,
          this.dateOfObjectCreation,
          this.dateOfLastOblectModification);
      this.record.setMetadata(idiomMets());
    } else if (this.getObjectType().startsWith("ConedaKorMediumData")) {

      System.out.println("  ++  id: " + id);
      System.out.println("  ++  tgBaseURI: " + textgridBaseURI_FromID);

      GetRecordType idi = new IDIOMImages().getRecordById(id);

      System.out.println(
          "  ++  idi: " + (idi == null ? "NULL" : idi.getRecord().getHeader().getIdentifier()));

      if (idi != null) {
        this.record = idi.getRecord();
      }

    }

    // See that IDs begin with "textgrid:"
    String identifierToSet = "";
    if (!id.startsWith("textgrid:")) {
      identifierToSet = "textgrid:" + id;
    } else {
      identifierToSet = id;
    }

    // See, that IDs have NO revision number.
    if (identifierToSet.contains(".")) {
      identifierToSet = identifierToSet.substring(0, identifierToSet.indexOf("."));
    }

    // No setSpec needed here!
    String setSpec = "";
    HeaderType header =
        OAIPMHUtilities.computeResponseHeader(this.dateOfObjectCreation, identifierToSet, setSpec);
    this.record.setHeader(header);

    grt.setRecord(this.record);

    return grt;
  }

  // **
  // PRIVATE METHODS
  // **

  /**
   * <p>
   * Getting the METS/MODS object for the ClassicMayan artifact and put them into metadata element
   * of the OAI-PMH response.
   * </p>
   * 
   * @return
   */
  private MetadataType idiomMets() {

    MetadataType metadataMets = new MetadataType();
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    DocumentBuilder builder;

    try {
      builder = factory.newDocumentBuilder();
      Document doc = builder.parse(new InputSource(new StringReader(this.metsmods.getMets())));
      metadataMets.setAny(doc.getDocumentElement());
    } catch (Exception e) {
      e.printStackTrace();
    }

    return metadataMets;
  }

  // **
  // GETTERS & SETTERS
  // **

  /**
   * @return
   */
  public String getObjectType() {
    return this.objectType;
  }

  /**
   * @param objectType
   */
  public void setObjectType(String objectType) {
    this.objectType = objectType;
  }

}
